function [link_order] = fun_analysis_get_capillary_order_to_labeled_vessels(vessel_graph, link_type_list, source_label, target_label)
% fun_analysis_get_capillary_order_to_labeled_vessels computes the geodesic
% distance between vessel segments of two different type labels. 
% Input: 
%   vessel_graph: structure generated by fun_skeleton_to_graph
%   link_type_list: numerical vector, type of the vessel link, should be of
%   the same length of the number of links in the vessel graph 
%   source_label: numerical scalar
%   target_label: numerical scalar
%
% Output: 
%   link_order: numerical vector of the same length as link_type_list. NaN
%   means that link is not connected to any links with target_label. 
% 
% Implemented by Xiang Ji on 07/27/2019

link_order = nan(vessel_graph.link.num_cc, 1);
% node_order = nan(vessel_graph.node.num_cc, 1);
current_link_label = find(link_type_list == target_label);
current_order = 1;
while ~isempty(current_link_label)
    % Connected node label
    tmp_connected_node = vessel_graph.link.connected_node_label(current_link_label, :);
    tmp_connected_node = unique(tmp_connected_node(tmp_connected_node > 0));
    % Connected link lable
    tmp_connected_link = cat(1, vessel_graph.node.connected_link_label{tmp_connected_node});
    % Determine what to label
    tmp_to_label_Q = (link_type_list(tmp_connected_link) == source_label);
    tmp_not_labeled_Q = isnan(link_order(tmp_connected_link));
    current_link_label = tmp_connected_link(tmp_to_label_Q & tmp_not_labeled_Q);
    link_order(current_link_label) = current_order;
    % Only label the capillary
%     tmp_is_capillary_Q_node = (tmp_connected_node) == unknown_label;
%     tmp_not_labeled_Q_node = isnan(node_order(tmp_connected_node));
%     tmp_connected_node = tmp_connected_node(tmp_not_labeled_Q_node & tmp_is_capillary_Q_node);
%     node_order(tmp_connected_node) = current_order;
    % Increase order
    current_order = current_order + 1;
end
%% Check why some of the links are not labeled
% debug_link_label = 3321;
% debug_connected_node_label = vessel_graph.link.connected_node_label(debug_link_label, :);
% debug_connected_link_label = cat(1, vessel_graph.node.connected_link_label{debug_connected_node_label}); % All of these links are not labeled
% 2045 has an order label, but did not propagate to 3321 through node 1765

end