function path_str = fun_analysis_get_path_to_neighbors_by_node_label(weighted_graph, node_label, neighbor_node_cell_array)
%fun_analysis_get_path_to_neighbors finds path to (nearest) neighbor node
%and compute the physical path length and geodesic path length. 
% Input: 
%   weighted_graph: weighted graph, one of the field of the graph structure
%   generated by fun_analysis_get_connectivity_graph
%   neighbor_node_cell_array: cell array, the i-th cell contains the label of
%   the neighbor node of the i-th node. This input should be 
%   vessel_graph.node.features.neighbor_node_label
% Output:
%   path_str: strcutre with fields. 
%
% Implemented by Xiang Ji on 02/26/2019
%
persistent tmp_template
if isempty(tmp_template)
    tmp_template = fun_initialized_structure_array_with_fieldname_list({'path_to_neighbor_length', ...
    'path_to_neighbor_geodesic_length', 'path_to_nearest_neighbor', ...
    'path_to_nearest_neighbor_length', 'path_to_nearest_neighbor_geodesic_length'});
end
path_str = tmp_template;
% num_cc = numel(node_label);

if isempty(node_label) || isempty(neighbor_node_cell_array)
    return;
end
num_node = numel(node_label);
assert(num_node == size(neighbor_node_cell_array, 1), 'Number of node label does not equal to the number of neighbor node label cell array');
neighbor_path_geodesic_length = cell(num_node, 1);
neighbor_path_length = cell(num_node, 1);
nearest_neighbor_path = cell(num_node, 1);
nearest_neighbor_path_length = nan(num_node, 1);
nearest_neighbor_path_geodesic_length = nan(num_node, 1);
num_node_in_graph = numnodes(weighted_graph);
for iter_node = 1 : num_node
    tmp_neighbor_node_label = neighbor_node_cell_array{iter_node};
    tmp_num_neighbor = numel(tmp_neighbor_node_label);
    tmp_node_label = node_label(iter_node);
    if tmp_num_neighbor > 0
        tmp_path_length_list = nan(tmp_num_neighbor, 1);
        tmp_path_geodesic_length_list = nan(tmp_num_neighbor, 1);
        for iter_neighbor = 1 : tmp_num_neighbor
            if (tmp_node_label <= num_node_in_graph) && (tmp_neighbor_node_label(iter_neighbor) <= num_node_in_graph)
                [tmp_node_path, tmp_path_length] = shortestpath(weighted_graph, ...
                    tmp_node_label, tmp_neighbor_node_label(iter_neighbor), 'Method', 'positive');
                tmp_num_node = numel(tmp_node_path);
            else
                tmp_num_node = 0;
            end
            if tmp_num_node > 0
                tmp_path_geodesic_length_list(iter_neighbor) = tmp_num_node - 1;
                tmp_path_length_list(iter_neighbor) = tmp_path_length;
                if iter_neighbor == 1
                    % Assume the first node is the nearest neighbor
                    nearest_neighbor_path{iter_node} = tmp_node_path;
                    nearest_neighbor_path_length(iter_node) = tmp_path_length;
                    nearest_neighbor_path_geodesic_length(iter_node) = tmp_path_geodesic_length_list(iter_neighbor);
                end                
            end
        end
        neighbor_path_geodesic_length{iter_node} = tmp_path_geodesic_length_list;
        neighbor_path_length{iter_node} = tmp_path_length_list;
    end    
end
path_str.path_to_neighbor_length = neighbor_path_length;
path_str.path_to_neighbor_geodesic_length = neighbor_path_geodesic_length;
path_str.path_to_nearest_neighbor = nearest_neighbor_path;
path_str.path_to_nearest_neighbor_length = nearest_neighbor_path_length;
path_str.path_to_nearest_neighbor_geodesic_length = nearest_neighbor_path_geodesic_length;
end