function self_loop_str = fun_graph_get_self_loops(input_graph)
% fun_grpah_get_self_loops finds the link segments that have ends attached
% to the same node connected component
% Input: 
%   input_graph: structure generated by fun_skeleton_to_graph
% Output: 
%   self_loop_str: structure with fields: 
%       is_self_loop_Q : N-by-1 logical vector. True if the link is a
%       self-loop
%       link_label: N-by-1 numerical vector, label of links that form the self-loop
%       num_voxel: N-by-1 numerical vector number of voxels in the link
%       connected components
%
% Written by Xiang Ji on Jan 29, 2019
% 
% self-loop is a link that does not have endpoint and attach to the same
% node at both ends. 
self_loop_str.is_self_loop_Q = all(input_graph.link.connected_node_label ~= 0, 2) & ...
    (input_graph.link.connected_node_label(:,1) == input_graph.link.connected_node_label(:,2));
% Due to the current skeleton to graph conversion, it is possible to create
% self-loop that has 

self_loop_str.link_label = find(self_loop_str.is_self_loop_Q);
% Number of voxels in these links
self_loop_str.num_voxel = input_graph.link.num_voxel_per_cc(self_loop_str.link_label);
end